#include <TinyGPS++.h>
#include <SoftwareSerial.h>

//Version 1.0
//Expected output
//01012017|121212|+23.123454|-23.234564|999|9999|999|99
//7|6|10|10|3|4|3|2 (52 characters)
//date|time|lat|lng|speed|altitude|accuracy(hdop)|# sats

static const int GPSRXPin = 2, GPSTXPin = 3;
static const uint32_t GPSBaud = 9600;
static const uint32_t BTBaud = 9600;
static uint32_t lastSent;
String outputString = "";
String inputString = "";         // a string to hold incoming data
boolean stringComplete = false;  // whether the string is complete
TinyGPSPlus gps;
SoftwareSerial GPSSerial(GPSRXPin, GPSTXPin);

void setup() {
  // put your setup code here, to run once:
  Serial.begin(BTBaud);
  while (!Serial) {
    ; //Wait for serial port to start up
  }
  //Change the Bluetooth adapter advertised name
  Serial.print("AT+NAMETIS-GPS");
  delay(500);
  //Make sure to clear AT command from serial buffer
  clearSerialBuffer();
  //Start up the software serial for the GPS unit
  GPSSerial.begin(GPSBaud);
}

void loop() {
  // put your main code here, to run repeatedly:
  while (GPSSerial.available())
  //Only send gps data every 2 seconds
    if (gps.encode(GPSSerial.read()) && (millis() - lastSent >= 2000)) {
      displayInfo();
      lastSent = millis();
    }
  if (stringComplete) {
    Serial.println(inputString);
    // clear the string:
    inputString = "";
    stringComplete = false;
  }
  //If we have not received any gps data for over 10 seconds
  //Print out an error
  if (millis() - lastSent > 10000)
  {
    Serial.print("Error|No GPS detected|unknown|unknown|0|0|0|0");
    Serial.print("\n");
    lastSent = millis();
  }
}

void displayInfo() {
  outputString = "";
  if (gps.date.isValid()) {
    // Make sure exactly 2 char, 2 char, 4 char
    outputString.concat(zeroPaddedValue(String(gps.date.month()), 2));
    outputString.concat(zeroPaddedValue(String(gps.date.day()), 2));
    outputString.concat(zeroPaddedValue(String(gps.date.year()), 4));
  } else {
    outputString.concat("INVALID");
  }

  outputString.concat("|");
  if (gps.time.isValid()) {
    // Make sure exactly 2 char, 2 char, 2 char
    outputString.concat(zeroPaddedValue(String(gps.time.hour()), 2));
    outputString.concat(zeroPaddedValue(String(gps.time.minute()), 2));
    outputString.concat(zeroPaddedValue(String(gps.time.second()), 2));
  } else {
    outputString.concat("INVALID");
  }
  //Serial.print("Location: ");
  outputString.concat("|");
  if (gps.location.isValid()) {
    outputString.concat(paddedValue(String(gps.location.lat(), 6), 10));
    outputString.concat("|");
    outputString.concat(paddedValue(String(gps.location.lng(), 6), 10));
  } else {
    outputString.concat("INVALID");
  }

  //Serial.print(" Speed: ");
  outputString.concat("|");
  if (gps.speed.isValid()) {
    outputString.concat(paddedValue(String(gps.speed.mph(), 0), 3));
  } else {
    outputString.concat("INVALID");
  }

  //Serial.print(" Altitude: ");
  outputString.concat("|");
  if (gps.altitude.isValid()) {
    outputString.concat(paddedValue(String(gps.altitude.feet(), 0), 4));
  } else {
    outputString.concat("INVALID");
  }

  //Serial.print(" HDOP: ");
  outputString.concat("|");
  if (gps.hdop.isValid()) {
    outputString.concat(paddedValue(String(gps.hdop.value()), 3));
  } else {
    outputString.concat("INVALID");
  }
  
  //Serial.print(" Satellites: ");
  outputString.concat("|");
  if (gps.satellites.isValid()) {
    outputString.concat(paddedValue(String(gps.satellites.value()), 2));
  } else {
    outputString.concat("INVALID");
  }
  Serial.print(outputString);
  Serial.print("\n");
}

String paddedValue(String inValue, int paddingLength) {
  while (inValue.length() < paddingLength) {
    inValue = " " + inValue;
  }
  return inValue;
}

String zeroPaddedValue(String inValue, int paddingLength) {
  while (inValue.length() < paddingLength) {
    inValue = "0" + inValue;
  }
  return inValue;
}

void serialEvent() {
  while (Serial.available()) {
    // get the new byte:
    char inChar = (char)Serial.read();
    // add it to the inputString:
    inputString += inChar;
    // if the incoming character is a newline, set a flag
    // so the main loop can do something about it:
    if (inChar == '\n') {
      stringComplete = true;
    }
  }
}

void clearSerialBuffer() {
  while (Serial.available() > 0) {
     char k = Serial.read();
  }
}

