package org.nissley.bluetooth_tester;

import android.app.Activity;
import android.bluetooth.BluetoothAdapter;
import android.bluetooth.BluetoothDevice;
import android.bluetooth.BluetoothManager;
import android.content.Context;
import android.content.Intent;
import android.os.Bundle;
import android.os.Handler;
import android.os.Message;
import android.support.v4.content.ContextCompat;
import android.support.v7.app.AppCompatActivity;
import android.util.Log;
import android.view.Menu;
import android.view.MenuItem;
import android.widget.ArrayAdapter;
import android.widget.LinearLayout;
import android.widget.ListView;
import android.widget.TextView;
import android.widget.Toast;

import java.util.ArrayList;

public class ScrollingActivity extends AppCompatActivity {

    private BluetoothAdapter myBluetoothAdapter;
    private static final int REQUEST_ENABLE_BT = 1;
    /** Tag for Log */
    private static final String TAG = "DeviceListActivity";

    /**
     * Member object for the services
     */
    private BluetoothService btService = null;
    private ArrayList<GPSRow> GPSRows;
    private ArrayAdapter<GPSRow> btMessageAdapter;
    private ListView messageView;
    private LinearLayout loStatus;
    private TextView connectStatus;

    @Override
    protected void onCreate(Bundle savedInstanceState) {
        super.onCreate(savedInstanceState);
        setContentView(R.layout.activity_scrolling);
        GPSRows = new ArrayList<>();
        final BluetoothManager btManager = (BluetoothManager)getSystemService(Context.BLUETOOTH_SERVICE);
        myBluetoothAdapter = btManager.getAdapter();
        if (myBluetoothAdapter == null) {
            Toast.makeText(getApplicationContext(),"Your device does not support Bluetooth",
                    Toast.LENGTH_LONG).show();
        } else {
            if (myBluetoothAdapter != null && !myBluetoothAdapter.isEnabled()) {
                Intent enableIntent = new Intent(BluetoothAdapter.ACTION_REQUEST_ENABLE);
                startActivityForResult(enableIntent, REQUEST_ENABLE_BT);
            }
        }
        btService = new BluetoothService(this, BTHandler);
        messageView = (ListView) findViewById(R.id.lvGPS1);
        //btMessageAdapter = new ArrayAdapter<String>(this, android.R.layout.simple_list_item_1);
        btMessageAdapter = new GPSRowArrayAdapter(this, 0, GPSRows);
        messageView.setAdapter(btMessageAdapter);
        loStatus = (LinearLayout) findViewById(R.id.lo_ConnectStatus);
        connectStatus = (TextView) findViewById(R.id.tv_ConnectStatustxt);
        loStatus.setBackgroundColor(ContextCompat.getColor(this, R.color.disconnected));
    }

    @Override
    protected void onDestroy() {
        super.onDestroy();
        btService.stop();
    }

    @Override
    public boolean onCreateOptionsMenu(Menu menu) {
        // Inflate the menu; this adds items to the action bar if it is present.
        getMenuInflater().inflate(R.menu.menu_scrolling, menu);
        return true;
    }

    @Override
    public boolean onOptionsItemSelected(MenuItem item) {
        // Handle action bar item clicks here. The action bar will
        // automatically handle clicks on the Home/Up button, so long
        // as you specify a parent activity in AndroidManifest.xml.
        switch (item.getItemId()) {
            case R.id.action_settings:
                return true;
            case R.id.mi_getDevice:
                Intent btDevicesIntent = new Intent(this, BTDeviceDialog.class);
                startActivityForResult(btDevicesIntent, 1);
                return true;
        }
        return super.onOptionsItemSelected(item);
    }

    @Override
    public void onActivityResult(int requestCode, int resultCode, Intent data) {
        super.onActivityResult(requestCode, resultCode, data);
        if (requestCode == 1) {
            if(resultCode == Activity.RESULT_OK){
                String macAddress = data.getStringExtra(BTDeviceDialog.EXTRA_DEVICE_ADDRESS);
                // Get the BluetoothDevice object
                BluetoothDevice device = myBluetoothAdapter.getRemoteDevice(macAddress);
                if (btService.getState() != BluetoothService.STATE_NONE) {
                    btService.stop();
                } else {
                    btService.connect(device);
                }
            }
            if (resultCode == Activity.RESULT_CANCELED) {
                //Write your code if there's no result
            }
        }
    }

    /**
     * The Handler that gets information back from the BluetoothChatService
     */
    private final Handler BTHandler = new Handler() {
        @Override
        public void handleMessage(Message msg) {
            switch (msg.what) {
                case Constants.MESSAGE_STATE_CHANGE:
                    switch (msg.arg1) {
                        case BluetoothService.STATE_CONNECTED:
                            Log.e(TAG, "State: Connected");
                            btMessageAdapter.clear();
                            loStatus.setBackgroundColor(ContextCompat.getColor(getApplicationContext(), R.color.connected));
                            connectStatus.setText("Connected");
                            break;
                        case BluetoothService.STATE_CONNECTING:
                            Log.e(TAG, "State: Connecting");
                            loStatus.setBackgroundColor(ContextCompat.getColor(getApplicationContext(), R.color.connecting));
                            connectStatus.setText("Connecting");
                            break;
                        case BluetoothService.STATE_NONE:
                            Log.e(TAG, "State: None");
                            loStatus.setBackgroundColor(ContextCompat.getColor(getApplicationContext(), R.color.disconnected));
                            connectStatus.setText("None");
                            break;
                        case BluetoothService.STATE_LOST_CONNECTION:
                            Log.e(TAG, "State: Lost Connection");
                            loStatus.setBackgroundColor(ContextCompat.getColor(getApplicationContext(), R.color.lost_connection));
                            connectStatus.setText("Lost Connection");
                            break;
                        case BluetoothService.STATE_CONNECTION_FAILED:
                            Log.e(TAG, "State: Connection Failed");
                            loStatus.setBackgroundColor(ContextCompat.getColor(getApplicationContext(), R.color.connection_failed));
                            connectStatus.setText("Connection Failed");
                            break;
                    }
                    break;
                case Constants.MESSAGE_READ:
                    // construct a string from the valid bytes in the buffer
                    String readMessage = msg.obj.toString();
                    Log.e(TAG, "State: Message Read");
                    btMessageAdapter.add(parseGPSData(readMessage));
                    //btMessageAdapter.add(new GPSRow("Date", "Time", 13, "93.2343", "-23.4333", 645, 26, 150));
                    //String sRetrunMsg = "Got it";
                    //btService.write(sRetrunMsg.getBytes());
                    break;
            }
        }
    };

    private GPSRow parseGPSData(String inputData) {
        String[] items = inputData.split("\\|");
        Integer speed = 0;
        Integer altitude = 0;
        Integer accuracy = 0;
        Integer sats = 0;
        String date = items[0].trim();
        String month = date.substring(0,2);
        String day = date.substring(2,4);
        String year = date.substring(4);
        date = month + "/" + day + "/" + year;
        String time = items[1].trim();
        String hours = time.substring(0,2);
        String minutes = time.substring(2,4);
        String seconds = time.substring(4);
        time = hours + ":" + minutes + ":" + seconds + " GMT";
        String lat = items[2].trim();
        String lng = items[3].trim();
        try {
            speed = Integer.parseInt(items[4].trim());
        } catch(NumberFormatException nfe) {
            speed = -1;
        }
        try {
            altitude = Integer.parseInt(items[5].trim());
        } catch(NumberFormatException nfe1) {
            altitude = -1;
        }
        try {
            accuracy = Integer.parseInt(items[6].trim());
        } catch(NumberFormatException nfe2) {
            accuracy = -1;
        }
        try {
            sats = Integer.parseInt(items[7].trim());
        } catch(NumberFormatException nfe3) {
            sats = -1;
        }
        GPSRow row = new GPSRow(date,time, sats, lat, lng, altitude, speed, accuracy);
        return row;
    };
}
